/*--------------------------------------------------------------------*/
/*    Copyright 2005 Sandia Corporation.                              */
/*    Under the terms of Contract DE-AC04-94AL85000, there is a       */
/*    non-exclusive license for use of this work by or on behalf      */
/*    of the U.S. Government.  Export of this program may require     */
/*    a license from the United States Government.                    */
/*--------------------------------------------------------------------*/

//
// This is a simple program to exercise FEI classes for the
// purposes of testing, code tuning and scaling studies.
//

#include "fei_iostream.hpp" 


#include <beam.hpp>
#include <Teuchos_ParameterList.hpp>

//fei_test_utils.hpp declares things like the intialize_mpi function.

#include "fei_test_utils.hpp"
#include <fei_ParameterSet.hpp>
#include <fei_utils.hpp>

void copy_fei_parameterset(const fei::ParameterSet& paramset,
                           Teuchos::ParameterList& paramlist);
//
//Include definitions of macros like 'CHK_ERR' to call functions and check
//the return code.
//
#undef fei_file
#define fei_file "beam_main.cpp"
#include "fei_ErrMacros.hpp"

//==============================================================================
//Here's the main...
//==============================================================================
int main(int argc, char** argv)
{
  MPI_Comm comm;
  int numProcs, localProc;
  CHK_ERR( fei_test_utils::initialize_mpi(argc, argv, localProc, numProcs) );
  comm = MPI_COMM_WORLD;

  //read input parameters from a file specified on the command-line with
  // '-i file' and optionally '-d path'
  std::vector<std::string> stdstrings;
  CHK_ERR( fei_test_utils::get_filename_and_read_input(argc, argv,
						comm, localProc,
						stdstrings) );
  fei::ParameterSet params;
  //parse the strings from the input file into a fei::ParameterSet object.
  fei::utils::parse_strings(stdstrings, " ", params);

  Teuchos::ParameterList parameterlist;
  copy_fei_parameterset(params, parameterlist);

  int errcode = beam(comm, parameterlist);

#ifndef FEI_SER
    MPI_Finalize();
#endif

  return(errcode);
}

void copy_fei_parameterset(const fei::ParameterSet& paramset,
                           Teuchos::ParameterList& paramlist)
{
  fei::ParameterSet::const_iterator
    iter = paramset.begin(),
    iter_end = paramset.end();

  for(; iter != iter_end; ++iter) {
    const fei::Param& param = *iter;
    fei::Param::ParamType ptype = param.getType();
    switch(ptype) {
    case fei::Param::STRING:
      paramlist.set(param.getName(), param.getStringValue());
      break;
    case fei::Param::DOUBLE:
      paramlist.set(param.getName(), param.getDoubleValue());
      break;
    case fei::Param::INT:
      paramlist.set(param.getName(), param.getIntValue());
      break;
    case fei::Param::BOOL:
      paramlist.set(param.getName(), param.getBoolValue());
      break;
    default:
      break;
    }
  }
}

