AC_DEFUN([BEAM_TRILINOS],
[
AC_ARG_WITH(trilinos-installdir,
AC_HELP_STRING([--with-trilinos-installdir],
[Specify location of Trilinos installation directory. NOTE: This is a directory produced by Trilinos' 'make install' target. It is assumed that the Trilinos build included FEI as well as Teuchos, Epetra, AztecOO, and Ifpack. configure will check whether other Trilinos libraries exist (libml.a, libamesos.a, libepetraext.a) and add them to the link line in the proper order.]),
[
AC_MSG_CHECKING(specified Trilinos installation-dir)
AC_MSG_RESULT([${withval}])
TRILINOS_INSTALL_DIR=${withval}
],
[
AC_MSG_RESULT([none])
]
)

save_CPPFLAGS=${CPPFLAGS}

if test "X${TRILINOS_INSTALL_DIR}" != "X"; then
CPPFLAGS="${CPPFLAGS} -I${TRILINOS_INSTALL_DIR}/include"
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_CPP( [#include <fei_base.hpp>],
[
TRILINOS_INCLUDE=-I${TRILINOS_INSTALL_DIR}/include
TRILINOS_LDFLAGS="-L${TRILINOS_INSTALL_DIR}/lib"
TRILINOS_FEI_LIBS="-lfei_trilinos -lfei_base"
TRILINOS_BASE_LIBS="-laztecoo -lepetra -lteuchos"
AC_DEFINE(BEAM_HAVE_TRILINOS,,[define if Trilinos available])
AM_CONDITIONAL(BEAM_HAVE_TRILINOS,true)
],
[
echo "!!!"
echo "Specified Trilinos installation directory ${TRILINOS_INSTALL_DIR}"
echo "doesn't seem to contain fei_base.hpp. (Meaning Trilinos wasn't configured"
echo "with '--enable-fei'."
echo "Trilinos support will not be enabled."
echo "!!!"
])
AC_LANG_RESTORE
else
AC_MSG_CHECKING(for --with-trilinos-installdir argument)
AC_MSG_RESULT([none])
TRILINOS_INCLUDE=-I.
fi

if test "X${TRILINOS_LDFLAGS}" = "X"; then
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_COMPILE([#include <Epetra_CrsMatrix.h>],[int dummy = 1; return(dummy);],
fei_have_trilinos=yes,fei_have_trilinos=no)
AC_LANG_RESTORE
if test ${fei_have_trilinos} = yes; then
AC_DEFINE(BEAM_HAVE_TRILINOS,,[define if Trilinos available])
AM_CONDITIONAL(BEAM_HAVE_TRILINOS,true)
else
AM_CONDITIONAL(BEAM_HAVE_TRILINOS,false)
fi
fi

if test "X${TRILINOS_BASE_LIBS}" != "X"; then
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_TRY_CPP([#include <EpetraExt_MatrixMatrix.h>],
[trilinos_epetraext=-lepetraext],[have_trilinos_epetraext=no])
AC_TRY_CPP([#include <Amesos.h>],
[trilinos_amesos=-lamesos],[have_trilinos_amesos=no])
AC_TRY_CPP([#include <ml_include.h>],
[trilinos_ml=-lml],[have_trilinos_ml=no])
AC_TRY_CPP([#include <Ifpack.h>],
[trilinos_ifpack=-lifpack],[have_trilinos_ifpack=no])

if test "X${trilinos_amesos}" != "X"; then
AC_DEFINE(BEAM_HAVE_TRILINOS_AMESOS,,[define if Trilinos/Amesos is available])
fi
if test "X${trilinos_ifpack}" != "X"; then
AC_DEFINE(BEAM_HAVE_TRILINOS_IFPACK,,[define if Trilinos/Ifpack is available])
fi
if test "X${trilinos_ml}" != "X"; then
AC_DEFINE(BEAM_HAVE_TRILINOS_ML,,[define if Trilinos/ML is available])
fi

TRILINOS_LIBS="${TRILINOS_FEI_LIBS} ${trilinos_ml} ${trilinos_ifpack} ${trilinos_amesos} ${trilinos_epetraext} ${TRILINOS_BASE_LIBS}"
else
TRILINOS_LIBS=${TRILINOS_FEI_LIBS} ${TRILINOS_BASE_LIBS}
fi

CPPFLAGS=${save_CPPFLAGS}

AC_SUBST(TRILINOS_INCLUDE)
AC_SUBST(TRILINOS_LDFLAGS)
AC_SUBST(TRILINOS_LIBS)
])
