#ifndef _hexbeam_functions_hpp_
#define _hexbeam_functions_hpp_

#include <utility>
#include <cmath>
#include "hexbeam_exception.hpp"
#include "HexBeam.hpp"
#include "fei_base.hpp"

namespace hexbeam {

std::pair<int,int>
compute_num_procs_2D(int numProcsTotal, int W, int D);

int print_beam_data(HexBeam& hexbeam, int numProcs, int localProc);

int init_elem_connectivities(fei::MatrixGraph* matrixGraph, HexBeam& hexbeam);

int init_shared_nodes(fei::MatrixGraph* matrixGraph, HexBeam& hexbeam);

int init_constraints(fei::MatrixGraph* matrixGraph, HexBeam& hexbeam,
		     int localProc, int& firstLocalCRID);

int init_slave_constraints(fei::MatrixGraph* matrixGraph, HexBeam& hexbeam, int dofPerNode);

/** Fill 'stiffness' with a dense, square, element-stiffness matrix.
    Stiffness matrix size is n X n where n is numNodes*dofPerNode.
    So on exit the length of the 'stiffness' array will be n*n.
*/
void elemStiffnessMatrix(int elemID,
                         int numNodes,
                         int dofPerNode,
                         std::vector<double>& stiffness);

/** On exit the length of the 'load' array will be n = numNodes*dofPerNode.
*/
void elemLoadVector(int elemID,
                    int numNodes,
                    int dofPerNode,
                    std::vector<double>& load);

int load_elem_data(fei::MatrixGraph* matrixGraph,
		   fei::Matrix* mat,
		   fei::Vector* rhs,
		   HexBeam& hexbeam,
                   int dofPerNode);

int load_constraints(fei::LinearSystem* linSys, HexBeam& hexbeam,
                     int dofPerNode,
		     int firstLocalCRID);

int load_BC_data(fei::LinearSystem* linSys, HexBeam& hexbeam, int dofPerNode);

}//namespace hexbeam

#endif

