#include "fei_iostream.hpp"
#include "utest_runner.hpp"

#include "utest_testcase0.hpp"
#include "utest_testcase1.hpp"
#include "utest_testcase2.hpp"
#include "utest_testcase3.hpp"
#include "utest_testcase4.hpp"

namespace beam_utest {

void register_tests(std::vector<testcase*>& tests)
{
  tests.clear();

  tests.push_back(new testcase0);
  tests.push_back(new testcase1);
  tests.push_back(new testcase2);
  tests.push_back(new testcase3);
  tests.push_back(new testcase4);
}

void destroy_tests(std::vector<testcase*>& tests)
{
  std::vector<testcase*>::iterator
    iter = tests.begin(), end = tests.end();

  for(; iter!=end; ++iter) {
    delete *iter;
  }
}
 
runner::runner()
 : all_tests()
{
  register_tests(all_tests);
}

runner::~runner()
{
  destroy_tests(all_tests);
}

void runner::run_tests(MPI_Comm comm)
{
  std::vector<testcase*>::iterator
    iter = all_tests.begin(), end = all_tests.end();

  try {

  for(; iter!=end; ++iter) {
    testcase& test = *(*iter);

    test.setup(comm);
    test.run(comm);
  }

  }
  catch(std::exception& exc) {
    FEI_COUT << "test failed: " << exc.what() << FEI_ENDL;
  }
}

}//namespace beam_utest

