#include "fei_iostream.hpp"

#include <sstream>

#include "utest_testcase0.hpp"
#include "HexBeam.hpp"
#include "hexbeam_exception.hpp"
#include "hexbeam_functions.hpp"

namespace beam_utest {

void check_compute_num_procs_2D(int numProcsTotal, int W, int D,
                                int expectWprocs, int expectDprocs,
                                bool expect_fail=false)
{
  bool threw_exception = false;

  std::pair<int,int> procs;

  try {
    procs = hexbeam::compute_num_procs_2D(numProcsTotal, W, D);

    if (procs.first != expectWprocs || procs.second != expectDprocs) {
      std::ostringstream osstr;
      osstr << "compute_num_procs_2D failed. numProcsTotal="<<numProcsTotal
        <<", W="<<W<<", D="<<D << ",   expectWprocs="<<expectWprocs
        <<", expectDprocs="<<expectDprocs <<"; W_procs="<<procs.first
        <<", D_procs="<<procs.second;
      std::string str = osstr.str();
      throw hexbeam::exception(str);
    }
  }
  catch(std::exception& exc) {
    threw_exception = true;
    if (!expect_fail) throw;
  }

  if (expect_fail && threw_exception == false) {
    std::ostringstream osstr;
      osstr << "compute_num_procs_2D failed. numProcsTotal="<<numProcsTotal
        <<", W="<<W<<", D="<<D << ",   expected exception-throw"
        <<"; produced W_procs="<<procs.first <<", D_procs="<<procs.second;
      std::string str = osstr.str();
      throw hexbeam::exception(str);
  }
}

void
check_coords(double x, double y, double z,
             double x_expected, double y_expected, double z_expected)
{
  if (std::abs(x - x_expected) > std::numeric_limits<double>::epsilon()) {
    throw hexbeam::exception("x coordinate is incorrect.");
  }
  if (std::abs(y - y_expected) > std::numeric_limits<double>::epsilon()) {
    throw hexbeam::exception("y coordinate is incorrect.");
  }
  if (std::abs(z - z_expected) > std::numeric_limits<double>::epsilon()) {
    throw hexbeam::exception("z coordinate is incorrect.");
  }
}

void
testcase0::run(MPI_Comm comm)
{
  FEI_COUT << "testcase0: testing hexbeam::compute_num_procs_2D...";

  int numProcsTotal = 100;
  int W = 10;
  int D = 10;

  check_compute_num_procs_2D(numProcsTotal, W, D, 10, 10);

  numProcsTotal = 1;

  check_compute_num_procs_2D(numProcsTotal, W, D, 1, 1);

  numProcsTotal = 100;
  W = 10;
  D = 100;

  check_compute_num_procs_2D(numProcsTotal, W, D, 2, 50);

  W = 5;
  D = 100;

  check_compute_num_procs_2D(numProcsTotal, W, D, 5, 20);

  numProcsTotal = 66;
  W = 4;
  D = 1000;

  check_compute_num_procs_2D(numProcsTotal, W, D, 2, 33);

  numProcsTotal = 512;
  W = 100;
  D = 100;

  check_compute_num_procs_2D(numProcsTotal, W, D, 0, 0, true);

  numProcsTotal = 67;
  W = 4;
  D = 1000;

  check_compute_num_procs_2D(numProcsTotal, W, D, 0, 0, true);

  FEI_COUT << "ok"<<FEI_ENDL;

  FEI_COUT << "testcase0: testing HexBeam::getNodeCoords...";

  HexBeam hexbeam(2, 2, HexBeam::OneD, 1, 0);

  double x, y, z;
  
  hexbeam.getNodeCoords(1, x, y, z);
  check_coords(x, y, z, 0.0, 0.0, 0.0);

  hexbeam.getNodeCoords(3, x, y, z);
  check_coords(x, y, z, 2.0, 0.0, 0.0);

  hexbeam.getNodeCoords(7, x, y, z);
  check_coords(x, y, z, 0.0, 2.0, 0.0);

  hexbeam.getNodeCoords(10, x, y, z);
  check_coords(x, y, z, 0.0, 0.0, 1.0);

  HexBeam hexbeam2(10, 2, HexBeam::OneD, 1, 0);

  hexbeam2.getNodeCoords(1, x, y, z);
  check_coords(x, y, z, 0.0, 0.0, 0.0);

  hexbeam2.getNodeCoords(15, x, y, z);
  check_coords(x, y, z, 3.0, 1.0, 0.0);

  hexbeam2.getNodeCoords(121, x, y, z);
  check_coords(x, y, z, 10.0, 10.0, 0.0);

  hexbeam2.getNodeCoords(122, x, y, z);
  check_coords(x, y, z, 0.0, 0.0, 1.0);

  hexbeam2.getNodeCoords(243, x, y, z);
  check_coords(x, y, z, 0.0, 0.0, 2.0);

  FEI_COUT << "ok"<<FEI_ENDL;
}

}//namespace beam_utest

