#include "fei_iostream.hpp"

#include <sstream>

#include "utest_testcase2.hpp"

#include "hexbeam_exception.hpp"
#include "hexbeam_functions.hpp"

namespace beam_utest {

void check_hexbeam_getLocalElemIDs(int W, int D, HexBeam::Decomposition decomp,
                          int numProcs, int localProc,
                          std::vector<int>& expect_elemIDs)
{
  HexBeam hexbeam(W, D, decomp, numProcs, localProc);

  std::vector<int> elemIDs;
  hexbeam.getLocalElemIDs(elemIDs);

  if (elemIDs != expect_elemIDs) {
    std::ostringstream osstr;
    osstr << "HexBeam("<<W<<","<<D<<","<<to_string(decomp)<<", "
        <<numProcs<<", "<<localProc
        <<").getLocalElemIDs(...) returned the wrong set of elem ids.\n"
      << "expected: ";
    for(unsigned i=0; i<expect_elemIDs.size(); ++i) {
      std::cout << expect_elemIDs[i] << " ";
    }
    std::cout << "\n returned: ";
    for(unsigned i=0; i<elemIDs.size(); ++i) {
      std::cout << elemIDs[i] << " ";
    }

    std::string str = osstr.str();
    throw hexbeam::exception(str);
  }
}

void
testcase2::run(MPI_Comm comm)
{
  FEI_COUT << "testcase2: testing HexBeam getLocalElemIDs...";

  int numProcs = 3;
  int localProc = 1;
  int W = 3;
  int D = 3;

  //for a "beam" of size 3x3x3 split in 1D on 3 processors:
  //each processor should have a 3x3x1 "slice" of elements
  //on proc 1 they should be elements 10 through 18

  std::vector<int> elemIDs(9);
  for(unsigned int i=10; i<=18; ++i) elemIDs[i-10] = i;

  check_hexbeam_getLocalElemIDs(W, D, HexBeam::OneD, numProcs, localProc,
                                elemIDs);

  numProcs = 9;

  //for the same "beam" split in 2D on 9 processors:
  //each processor should have a 3x1x1 mini-beam of elements
  //on proc 1 the elements should be 4,5,6.
  elemIDs.resize(3);
  for(unsigned int i=4; i<=6; ++i) elemIDs[i-4] = i;

  check_hexbeam_getLocalElemIDs(W, D, HexBeam::TwoD, numProcs, localProc,
                                elemIDs);

  FEI_COUT << "ok"<<FEI_ENDL;
}

}//namespace beam_utest

