#include "fei_iostream.hpp"

#include <sstream>

#include "utest_testcase3.hpp"

#include "hexbeam_exception.hpp"
#include "hexbeam_functions.hpp"

namespace beam_utest {

void check_hexbeam_getLocalNodeIDs(int W, int D, HexBeam::Decomposition decomp,
                          int numProcs, int localProc,
                          std::vector<int>& expect_nodeIDs)
{
  HexBeam hexbeam(W, D, decomp, numProcs, localProc);

  std::vector<int> nodeIDs;
  hexbeam.getLocalNodeIDs(nodeIDs);

  if (nodeIDs != expect_nodeIDs) {
    std::ostringstream osstr;
    osstr << "HexBeam("<<W<<","<<D<<","<<to_string(decomp)<<", "
        <<numProcs<<", "<<localProc
        <<").getLocalNodeIDs(...) returned the wrong set of node ids.\n"
      << "expected: ";
    for(unsigned i=0; i<expect_nodeIDs.size(); ++i) {
      std::cout << expect_nodeIDs[i] << " ";
    }
    std::cout << "\n returned: ";
    for(unsigned i=0; i<nodeIDs.size(); ++i) {
      std::cout << nodeIDs[i] << " ";
    }

    std::string str = osstr.str();
    throw hexbeam::exception(str);
  }
}

void
testcase3::run(MPI_Comm comm)
{
  FEI_COUT << "testcase3: testing HexBeam getLocalNodeIDs...";

  int numProcs = 3;
  int localProc = 1;
  int W = 3;
  int D = 3;

  //for a "beam" of size 3x3x3 split in 1D on 3 processors:
  //each processor should have a 3x3x1 "slice" of elements
  //on proc 1 there should be 32 nodes, and they should be nodes 17 through 48

  std::vector<int> nodeIDs(32);
  for(unsigned int i=17; i<=48; ++i) nodeIDs[i-17] = i;

  check_hexbeam_getLocalNodeIDs(W, D, HexBeam::OneD, numProcs, localProc,
                                nodeIDs);

  numProcs = 9;

  //for the same "beam" split in 2D on 9 processors:
  //each processor should have a 3x1x1 mini-beam of elements
  //on proc 1 there should be 16 nodes: nodes 5,6,7,8, 9,10,11,12
  //                                          21,22,23,24, 25,26,27,28
  nodeIDs.resize(16);
  unsigned offset = 0;
  for(unsigned int i=5; i<=12; ++i) nodeIDs[offset++] = i;
  for(unsigned int i=21; i<=28; ++i) nodeIDs[offset++] = i;

  check_hexbeam_getLocalNodeIDs(W, D, HexBeam::TwoD, numProcs, localProc,
                                nodeIDs);

  FEI_COUT << "ok"<<FEI_ENDL;
}

}//namespace beam_utest

